==> rospell-2007.02.rc1.tar.gz: rospell-2007.02.rc1/src/rospell/spell.cpp <==
//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "spell.h"


bool hasnumeric(QString str) {
	int len = str.length();
	for (int i = 0; i < len; i++) {
		QChar ch = str[i];
		if (ch.isDigit())
			return true;
	}
	return false;
}

char *spell(Dictionaries *pdict, Action *aptr, char *fstr) {
	PROFILE("spell");
	ASSERT(pdict != 0);
	ASSERT(fstr != 0);
	ASSERT(aptr != 0);
	LOG("length %d\n", strlen(fstr));

	// get stats pointer
	Stats *pstats = Stats::instance();

	DbDict *hptr = pdict->getMaster();
	DbUser *uptr = pdict->getUser();
	DbUser *iptr = pdict->getIgnore();
	DbUser *fptr = pdict->getFixed();

	QString qs = QString::fromUtf8(fstr); // worning! it will not work in utf8 without it
	QChar qc = qs[0];
	int i1 = 0;
	int i2 = 0;
	while (!qs.isEmpty()) {
		// advance to next word
		while (!qc.isNull() && !qc.isLetterOrNumber()) {
			i1++;
			qc = qs[i1];
		}
		if (qc.isNull())
			return fstr;
		i2 = i1;

		// extract the word
		while (qc.isLetterOrNumber() || qc == '-') {
			i2++;
			qc = qs[i2];
		}
		QString extracted = qs.mid(i1, i2 - i1);
		i1 = i2;

		Q3CString utext = extracted.utf8();
		const char *a = utext;
		char *active_word = (char *) a;
		if (!hptr->findString(active_word) &&	// if not in dictionary
		    !fptr->findString(active_word, true) &&	 // if not in fixed/names dictionary
		    !uptr->findString(active_word) &&	// if not in user dictionary
		    !iptr->findString(active_word) &&	// if not in ignore dictionary
		    !hasnumeric(extracted)) {		 // is not a number

			pstats->notFound();

			if ((*aptr)(pdict, active_word)) {
				delete[] active_word;
				break;
			}
		} else
			pstats->found();


	}
	return fstr;
}

void readHomeDir(Dictionaries *pdict) {
	if (pdict == 0)
		return;
	// find user file
	// save new dictionary
	char *homedir = getenv("HOME");
	if (homedir == 0) {
		fprintf(stderr, "Error: HOME environment variable not found\n");
		exit(1);
	}
	unsigned len = strlen(homedir) + strlen("/.rospell/dictionary.fixed");
	char *fname = new char[len + 1];
	memset(fname, '\0', len + 1);
	strcpy(fname, homedir);
	strcat(fname, "/.rospell/dictionary.fixed");
	char *fstr = UTILS::read_file_malloc(fname);
	if (fstr != 0) {
		Action *aptr;
		aptr = new ActionAddAllFixed();
		spell(pdict, aptr, fstr);
		delete aptr;
	} else {
		Global::buildDirs();
	}

	delete[] fname;
	delete[] fstr;

	len = strlen(homedir) + strlen("/.rospell/dictionary.user");
	fname = new char[len + 1];
	memset(fname, '\0', len + 1);
	strcpy(fname, homedir);
	strcat(fname, "/.rospell/dictionary.user");
	fstr = UTILS::read_file_malloc(fname);
	if (fstr != 0) {
		Action *aptr;
		aptr = new ActionAddAllUser();
		spell(pdict, aptr, fstr);
		delete aptr;
	}
	delete[] fname;
	delete[] fstr;

	Stats::instance()->reset();
	LOG("HOME file was read\n");
}

==> rospell-2009.00-rc2.tar.gz: rospell-2009.00-rc2/src/rospell/spell.cpp <==
//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "spell.h"
#ifdef USING_QT4
#include <Q3CString>
#endif

bool hasnumeric(QString str) {
	int len = str.length();
	for (int i = 0; i < len; i++) {
		QChar ch = str[i];
		if (ch.isDigit())
			return true;
	}
	return false;
}

char *spell(Dictionaries *pdict, Action *aptr, char *fstr) {
	PROFILE("spell");
	ASSERT(pdict != 0);
	ASSERT(fstr != 0);
	ASSERT(aptr != 0);

	// get stats pointer
	Stats *pstats = Stats::instance();

	QString qs = QString::fromUtf8(fstr); // warning! it will not work in utf8 without it
	QChar qc = qs[0];
	int i1 = 0;
	int i2 = 0;
	while (!qs.isEmpty()) {
		// advance to next word
		while (!qc.isNull() && !qc.isLetterOrNumber()) {
			i1++;
			qc = qs[i1];
		}
		if (qc.isNull())
			return fstr;
		i2 = i1;

		// extract the word
		while (qc.isLetterOrNumber() || qc == '-') {
			i2++;
			qc = qs[i2];
		}
		QString extracted = qs.mid(i1, i2 - i1);
		i1 = i2;

		Q3CString utext = extracted.utf8();
		const char *a = utext;
		char *active_word = (char *) a;
		if (!pdict->findString(active_word) &&	// if not in dictionary
		    !hasnumeric(extracted)) {		 // is not a number

			pstats->notFound();

			if ((*aptr)(pdict, active_word)) {
				delete[] active_word;
				break;
			}
		} else
			pstats->found();


	}
	return fstr;
}
